@@ -19,6 +19,7 @@ from ..grids.base import DimensionError, GridBase
 from ..tools.docstrings import fill_in_docstring
 from ..tools.misc import get_common_dtype
 from ..tools.numba import get_common_numba_dtype
+from ..tools.typing import NumberOrArray
 from .base import DataFieldBase
 from .scalar import ScalarField
 
@@ -110,7 +111,7 @@ class VectorField(DataFieldBase):
 
         if isinstance(expressions, str) or len(expressions) != grid.dim:
             axes_names = grid.axes + grid.axes_symmetric
-            raise ValueError(
+            raise DimensionError(
                 f"Expected {grid.dim} expressions for the coordinates {axes_names}."
             )
 
@@ -129,11 +130,20 @@ class VectorField(DataFieldBase):
             grid=grid, data=data, label=label, dtype=dtype
         )
 
-    def __getitem__(self, key: int) -> ScalarField:
+    def __getitem__(self, key: Union[int, str]) -> ScalarField:
         """extract a component of the VectorField"""
-        if not isinstance(key, int):
-            raise IndexError("Index must be an integer")
-        return ScalarField(self.grid, self.data[key])
+        return ScalarField(self.grid, self.data[self.grid.get_axis_index(key)])
+
+    def __setitem__(
+        self, key: Union[int, str], value: Union[NumberOrArray, ScalarField]
+    ):
+        """set a component of the VectorField"""
+        idx = self.grid.get_axis_index(key)
+        if isinstance(value, ScalarField):
+            self.grid.assert_grid_compatible(value.grid)
+            self.data[idx] = value.data
+        else:
+            self.data[idx] = value
 
     def dot(
         self,
@@ -326,32 +336,6 @@ class VectorField(DataFieldBase):
 
         return dot
 
-    def get_dot_operator(self) -> Callable:
-        """return operator calculating the dot product involving vector fields
-
-        This supports both products between two vectors as well as products
-        between a vector and a tensor.
-
-        Warning:
-            This function does not check types or dimensions.
-
-        Returns:
-            function that takes two instance of :class:`~numpy.ndarray`, which
-            contain the discretized data of the two operands. An optional third
-            argument can specify the output array to which the result is
-            written. Note that the returned function is jitted with numba for
-            speed.
-        """
-        # Deprecated this method on 2020-10-07
-        import warnings
-
-        warnings.warn(
-            "get_dot_operator() method is deprecated. Use the `make_dot_operator()` "
-            "method instead",
-            DeprecationWarning,
-        )
-        return self.make_dot_operator()
-
     def outer_product(
         self, other: "VectorField", out: "Tensor2Field" = None, *, label: str = None
     ) -> "Tensor2Field":
@@ -498,13 +482,7 @@ class VectorField(DataFieldBase):
             :class:`~pde.fields.scalar.ScalarField`: result of applying the operator
         """
         divergence = self.grid.get_operator("divergence", bc=bc)
-        if out is None:
-            out = ScalarField(self.grid, divergence(self.data), label=label)
-        else:
-            assert isinstance(out, ScalarField), f"`out` must be ScalarField"
-            self.grid.assert_grid_compatible(out.grid)
-            divergence(self.data, out=out.data)
-        return out
+        return self._apply_with_out(divergence, ScalarField, out=out, label=label)
 
     @fill_in_docstring
     def gradient(
@@ -514,7 +492,14 @@ class VectorField(DataFieldBase):
         *,
         label: str = "gradient",
     ) -> "Tensor2Field":
-        """apply vector gradient operator and return result as a field
+        r"""apply vector gradient operator and return result as a field
+
+        The vector gradient field is a tensor field :math:`t_{\alpha\beta}` that
+        specifies the derivatives of the vector field :math:`v_\alpha` with respect to
+        all coordinates :math:`x_\beta`:
+
+        .. math::
+            t_{\alpha\beta} = \frac{\partial v_\alpha}{\partial x_\beta}
 
         Args:
             bc:
@@ -531,13 +516,7 @@ class VectorField(DataFieldBase):
         from .tensorial import Tensor2Field  # @Reimport
 
         vector_gradient = self.grid.get_operator("vector_gradient", bc=bc)
-        if out is None:
-            out = Tensor2Field(self.grid, vector_gradient(self.data), label=label)
-        else:
-            assert isinstance(out, Tensor2Field), f"`out` must be Tensor2Field"
-            self.grid.assert_grid_compatible(out.grid)
-            vector_gradient(self.data, out=out.data)
-        return out
+        return self._apply_with_out(vector_gradient, Tensor2Field, out=out, label=label)
 
     @fill_in_docstring
     def laplace(
@@ -547,7 +526,15 @@ class VectorField(DataFieldBase):
         *,
         label: str = "vector laplacian",
     ) -> "VectorField":
-        """apply vector Laplace operator and return result as a field
+        r"""apply vector Laplace operator and return result as a field
+
+        The vector Laplacian is a vector field :math:`L_\alpha` containing the second
+        derivatives of the vector field :math:`v_\alpha` with respect to the coordinates
+        :math:`x_\beta`:
+
+        .. math::
+            L_\alpha = \sum_\beta
+                \frac{\partial^2 v_\alpha}{\partial x_\beta \partial x_\beta}
 
         Args:
             bc:
@@ -561,10 +548,8 @@ class VectorField(DataFieldBase):
         Returns:
             :class:`~pde.fields.vectorial.VectorField`: result of applying the operator
         """
-        if out is not None:
-            assert isinstance(out, VectorField), f"`out` must be VectorField"
         laplace = self.grid.get_operator("vector_laplace", bc=bc)
-        return self.apply(laplace, out=out, label=label)
+        return self._apply_with_out(laplace, VectorField, out=out, label=label)
 
     @property
     def integral(self) -> np.ndarray:
