@@ -61,7 +61,7 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
 
     @property
     def cell_volume_data(self):
-        """ size associated with each cell """
+        """size associated with each cell"""
         return tuple(self.discretization)
 
     def contains_point(self, point: np.ndarray) -> np.ndarray:
@@ -104,7 +104,10 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
                 yield point + offset
 
     def get_random_point(
-        self, boundary_distance: float = 0, cartesian: bool = True
+        self,
+        boundary_distance: float = 0,
+        cartesian: bool = True,
+        rng: np.random.Generator = None,
     ) -> np.ndarray:
         """return a random point within the grid
 
@@ -115,10 +118,15 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
                 Cartesian coordinates or grid coordinates. This does not have
                 any effect for Cartesian coordinate systems, but the argument is
                 retained to have a unified interface for all grids.
+            rng (:class:`~numpy.random.Generator`):
+                Random number generator (default: :func:`~numpy.random.default_rng()`)
 
         Returns:
             :class:`~numpy.ndarray`: The coordinates of the point
         """
+        if rng is None:
+            rng = np.random.default_rng()
+
         # handle the boundary distance
         cuboid = self.cuboid
         if boundary_distance != 0:
@@ -127,7 +135,7 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
             cuboid = cuboid.buffer(-boundary_distance)
 
         # create random point
-        return cuboid.pos + np.random.random(self.dim) * cuboid.size  # type: ignore
+        return cuboid.pos + rng.random(self.dim) * cuboid.size  # type: ignore
 
     def get_line_data(self, data: np.ndarray, extract: str = "auto") -> Dict[str, Any]:
         """return a line cut through the given data
@@ -157,7 +165,7 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
             )
 
         def _get_axis(axis):
-            """ determine the axis from a given specifier """
+            """determine the axis from a given specifier"""
             try:
                 axis = int(axis)
             except ValueError:
@@ -443,7 +451,7 @@ class UnitGrid(CartesianGridBase):
 
     @property
     def state(self) -> Dict[str, Any]:
-        """ dict: the state of the grid """
+        """dict: the state of the grid"""
         return {"shape": self.shape, "periodic": self.periodic}
 
     @classmethod
@@ -462,7 +470,7 @@ class UnitGrid(CartesianGridBase):
 
     @property
     def volume(self) -> float:
-        """ float: total volume of the grid """
+        """float: total volume of the grid"""
         return float(np.prod(self.shape))
 
     def cell_to_point(self, cells: np.ndarray, cartesian: bool = True) -> np.ndarray:
@@ -524,7 +532,7 @@ class UnitGrid(CartesianGridBase):
         return diff  # type: ignore
 
     def to_cartesian(self) -> "CartesianGrid":
-        """ convert unit grid to CartesianGrid """
+        """convert unit grid to CartesianGrid"""
         return CartesianGrid(
             self.cuboid.bounds, shape=self.shape, periodic=self.periodic
         )
@@ -643,7 +651,7 @@ class CartesianGrid(CartesianGridBase):
 
     @property
     def state(self) -> Dict[str, Any]:
-        """ dict: the state of the grid """
+        """dict: the state of the grid"""
         return {
             "bounds": self.axes_bounds,
             "shape": self.shape,
@@ -670,7 +678,7 @@ class CartesianGrid(CartesianGridBase):
 
     @property
     def volume(self) -> float:
-        """ float: total volume of the grid """
+        """float: total volume of the grid"""
         return float(self.cuboid.volume)
 
     def cell_to_point(self, cells: np.ndarray, cartesian: bool = True) -> np.ndarray:
