@@ -45,7 +45,7 @@ def make_laplace(bcs: Boundaries) -> OperatorType:
     dim_r, dim_z = bcs.grid.shape
     dr_2, dz_2 = 1 / bcs.grid.discretization ** 2
 
-    value_outer = boundary_r.high.make_virtual_point_evaluator()
+    value_r_outer = boundary_r.high.make_virtual_point_evaluator()
     region_z = boundary_z.make_region_evaluator()
 
     # use processing for large enough arrays
@@ -78,7 +78,7 @@ def make_laplace(bcs: Boundaries) -> OperatorType:
             # outer radial boundary condition
             i = dim_r - 1
             arr_z_l, arr_c, arr_z_h = region_z(arr, (i, j))
-            arr_r_l, arr_r_h = arr[i - 1, j], value_outer(arr, (i, j))
+            arr_r_l, arr_r_h = arr[i - 1, j], value_r_outer(arr, (i, j))
             out[i, j] = (
                 (arr_r_h - 2 * arr_c + arr_r_l) * dr_2
                 + (arr_r_h - arr_r_l) / (2 * i + 1) * dr_2
@@ -111,7 +111,7 @@ def make_gradient(bcs: Boundaries) -> OperatorType:
     dim_r, dim_z = bcs.grid.shape
     scale_r, scale_z = 1 / (2 * bcs.grid.discretization)
 
-    value_outer = boundary_r.high.make_virtual_point_evaluator()
+    value_r_outer = boundary_r.high.make_virtual_point_evaluator()
     region_z = boundary_z.make_region_evaluator()
 
     # use processing for large enough arrays
@@ -137,7 +137,7 @@ def make_gradient(bcs: Boundaries) -> OperatorType:
             # outer radial boundary condition
             i = dim_r - 1
             arr_z_l, _, arr_z_h = region_z(arr, (i, j))
-            arr_r_h = value_outer(arr, (i, j))
+            arr_r_h = value_r_outer(arr, (i, j))
             out[0, i, j] = (arr_r_h - arr[i - 1, j]) * scale_r
             out[1, i, j] = (arr_z_h - arr_z_l) * scale_z
             out[2, i, j] = 0  # no phi dependence by definition
@@ -173,7 +173,7 @@ def make_gradient_squared(bcs: Boundaries, central: bool = True) -> OperatorType
     # calculate preliminary quantities
     dim_r, dim_z = bcs.grid.shape
 
-    value_outer = boundary_r.high.make_virtual_point_evaluator()
+    value_r_outer = boundary_r.high.make_virtual_point_evaluator()
     region_z = boundary_z.make_region_evaluator()
 
     # use processing for large enough arrays
@@ -203,7 +203,7 @@ def make_gradient_squared(bcs: Boundaries, central: bool = True) -> OperatorType
                 # outer radial boundary condition
                 i = dim_r - 1
                 arr_z_l, _, arr_z_h = region_z(arr, (i, j))
-                arr_r_h = value_outer(arr, (i, j))
+                arr_r_h = value_r_outer(arr, (i, j))
                 term_r = (arr_r_h - arr[i - 1, j]) ** 2
                 term_z = (arr_z_h - arr_z_l) ** 2
                 out[i, j] = term_r * scale_r + term_z * scale_z
@@ -234,7 +234,7 @@ def make_gradient_squared(bcs: Boundaries, central: bool = True) -> OperatorType
                 # outer radial boundary condition
                 i = dim_r - 1
                 arr_z_l, arr_c, arr_z_h = region_z(arr, (i, j))
-                arr_r_h = value_outer(arr, (i, j))
+                arr_r_h = value_r_outer(arr, (i, j))
                 term_r = (arr_r_h - arr_c) ** 2 + (arr_c - arr[i - 1, j]) ** 2
                 term_z = (arr_z_h - arr_c) ** 2 + (arr_c - arr_z_l) ** 2
                 out[i, j] = term_r * scale_r + term_z * scale_z
@@ -264,11 +264,9 @@ def make_divergence(bcs: Boundaries) -> OperatorType:
 
     # calculate preliminary quantities
     dim_r, dim_z = bcs.grid.shape
-    dr = bcs.grid.discretization[0]
-    scale_r, scale_z = 1 / (2 * bcs.grid.discretization)
-
-    value_outer = boundary_r.high.make_virtual_point_evaluator()
-    region_z = boundary_z.make_region_evaluator()
+    rs = bcs.grid.axes_coords[0]
+    der_r = boundary_r.make_derivative_evaluator()
+    der_z = boundary_z.make_derivative_evaluator()
 
     # use processing for large enough arrays
     parallel = dim_r * dim_z >= config["numba.parallel_threshold"]
@@ -276,29 +274,13 @@ def make_divergence(bcs: Boundaries) -> OperatorType:
     @jit_allocate_out(parallel=parallel, out_shape=(dim_r, dim_z))
     def divergence(arr, out=None):
         """apply divergence operator to array `arr`"""
-        for j in nb.prange(0, dim_z):  # iterate axial points
-            # inner radial boundary condition
-            i = 0
-            arr_z_l, _, arr_z_h = region_z(arr[1], (i, j))
-            d_r = (arr[0, 1, j] + 3 * arr[0, 0, j]) * scale_r
-            d_z = (arr_z_h - arr_z_l) * scale_z
-            out[i, j] = d_r + d_z
+        arr_r, arr_z = arr[0], arr[1]
 
-            for i in range(1, dim_r - 1):  # iterate radial points
-                arr_z_l, _, arr_z_h = region_z(arr[1], (i, j))
-                d_r = (arr[0, i + 1, j] - arr[0, i - 1, j]) * scale_r
-                d_r += arr[0, i, j] / ((i + 0.5) * dr)
-                d_z = (arr_z_h - arr_z_l) * scale_z
-                out[i, j] = d_r + d_z
-
-            # outer radial boundary condition
-            i = dim_r - 1
-            arr_z_l, _, arr_z_h = region_z(arr[1], (i, j))
-            arr_r_h = value_outer(arr[0], (i, j))
-            d_r = (arr_r_h - arr[0, i - 1, j]) * scale_r
-            d_r += arr[0, i, j] / ((i + 0.5) * dr)
-            d_z = (arr_z_h - arr_z_l) * scale_z
-            out[i, j] = d_z + d_r
+        for j in nb.prange(dim_z):  # iterate axial points
+            for i in range(dim_r):  # iterate radial points
+                out[i, j] = (
+                    arr_r[i, j] / rs[i] + der_z(arr_z, (i, j)) + der_r(arr_r, (i, j))
+                )
 
         return out
 
@@ -323,16 +305,44 @@ def make_vector_gradient(bcs: Boundaries) -> OperatorType:
     bcs.check_value_rank(1)
 
     # calculate preliminary quantities
-    gradient_r = make_gradient(bcs.extract_component(0))
-    gradient_z = make_gradient(bcs.extract_component(1))
-    gradient_phi = make_gradient(bcs.extract_component(2))
+    dim_r, dim_z = bcs.grid.shape
+    rs = bcs.grid.axes_coords[0]
 
-    @jit_allocate_out(out_shape=(3, 3) + bcs.grid.shape)
+    # handle the boundary conditions
+    boundary_r, boundary_z = bcs
+    deriv_r_r = boundary_r.extract_component(0).make_derivative_evaluator()
+    deriv_r_z = boundary_r.extract_component(1).make_derivative_evaluator()
+    deriv_r_φ = boundary_r.extract_component(2).make_derivative_evaluator()
+    deriv_z_r = boundary_z.extract_component(0).make_derivative_evaluator()
+    deriv_z_z = boundary_z.extract_component(1).make_derivative_evaluator()
+    deriv_z_φ = boundary_z.extract_component(2).make_derivative_evaluator()
+
+    # use processing for large enough arrays
+    parallel = dim_r * dim_z >= config["numba.parallel_threshold"]
+
+    @jit_allocate_out(parallel=parallel, out_shape=(3, 3) + bcs.grid.shape)
     def vector_gradient(arr, out=None):
         """apply gradient operator to array `arr`"""
-        gradient_r(arr[0], out=out[:, 0])
-        gradient_z(arr[1], out=out[:, 1])
-        gradient_phi(arr[2], out=out[:, 2])
+        # assign aliases
+        arr_r, arr_z, arr_φ = arr
+        out_rr, out_rz, out_rφ = out[0, 0], out[0, 1], out[0, 2]
+        out_zr, out_zz, out_zφ = out[1, 0], out[1, 1], out[1, 2]
+        out_φr, out_φz, out_φφ = out[2, 0], out[2, 1], out[2, 2]
+
+        for j in nb.prange(dim_z):  # iterate axial points
+            for i in range(dim_r):  # iterate radial points
+                out_rr[i, j] = deriv_r_r(arr_r, (i, j))
+                out_φr[i, j] = deriv_r_φ(arr_φ, (i, j))
+                out_zr[i, j] = deriv_r_z(arr_z, (i, j))
+
+                out_rφ[i, j] = -arr_φ[i, j] / rs[i]
+                out_φφ[i, j] = arr_r[i, j] / rs[i]
+                out_zφ[i, j] = 0
+
+                out_rz[i, j] = deriv_z_r(arr_r, (i, j))
+                out_φz[i, j] = deriv_z_φ(arr_φ, (i, j))
+                out_zz[i, j] = deriv_z_z(arr_z, (i, j))
+
         return out
 
     return vector_gradient  # type: ignore
@@ -355,16 +365,56 @@ def make_vector_laplace(bcs: Boundaries) -> OperatorType:
     assert isinstance(bcs.grid, CylindricalSymGrid)
     bcs.check_value_rank(1)
 
-    laplace_r = make_laplace(bcs.extract_component(0))
-    laplace_z = make_laplace(bcs.extract_component(1))
-    laplace_phi = make_laplace(bcs.extract_component(2))
+    # calculate preliminary quantities
+    dim_r, dim_z = bcs.grid.shape
+    rs = bcs.grid.axes_coords[0]
+    dr = bcs.grid.discretization[0]
+    s1, s2 = 1 / (2 * dr), 1 / dr ** 2
 
-    @jit_allocate_out(out_shape=(3,) + bcs.grid.shape)
+    # handle the boundary conditions
+    boundary_r, boundary_z = bcs
+    region_r_r = boundary_r.extract_component(0).make_region_evaluator()
+    region_r_z = boundary_r.extract_component(1).make_region_evaluator()
+    region_r_φ = boundary_r.extract_component(2).make_region_evaluator()
+    deriv_zz_r = boundary_z.extract_component(0).make_derivative_evaluator(order=2)
+    deriv_zz_z = boundary_z.extract_component(1).make_derivative_evaluator(order=2)
+    deriv_zz_φ = boundary_z.extract_component(2).make_derivative_evaluator(order=2)
+
+    # use processing for large enough arrays
+    parallel = dim_r * dim_z >= config["numba.parallel_threshold"]
+
+    @jit_allocate_out(parallel=parallel, out_shape=(3,) + bcs.grid.shape)
     def vector_laplace(arr, out=None):
-        """apply gradient operator to array `arr`"""
-        laplace_r(arr[0], out=out[0])
-        laplace_z(arr[1], out=out[1])
-        laplace_phi(arr[2], out=out[2])
+        """apply vector laplace operator to array `arr`"""
+        # assign aliases
+        arr_r, arr_z, arr_φ = arr
+        out_r, out_z, out_φ = out
+
+        for j in nb.prange(dim_z):  # iterate axial points
+            for i in range(dim_r):  # iterate radial points
+                f_r_l, f_r_m, f_r_h = region_r_r(arr_r, (i, j))
+                out_r[i, j] = (
+                    deriv_zz_r(arr_r, (i, j))
+                    - f_r_m / rs[i] ** 2
+                    + (f_r_h - f_r_l) * s1 / rs[i]
+                    + (f_r_h - 2 * f_r_m + f_r_l) * s2
+                )
+
+                f_φ_l, f_φ_m, f_φ_h = region_r_φ(arr_φ, (i, j))
+                out_φ[i, j] = (
+                    deriv_zz_φ(arr_φ, (i, j))
+                    - f_φ_m / rs[i] ** 2
+                    + (f_φ_h - f_φ_l) * s1 / rs[i]
+                    + (f_φ_h - 2 * f_φ_m + f_φ_l) * s2
+                )
+
+                f_z_l, f_z_m, f_z_h = region_r_z(arr_z, (i, j))
+                out_z[i, j] = (
+                    deriv_zz_z(arr_z, (i, j))
+                    + (f_z_h - f_z_l) * s1 / rs[i]
+                    + (f_z_h - 2 * f_z_m + f_z_l) * s2
+                )
+
         return out
 
     return vector_laplace  # type: ignore
@@ -387,16 +437,42 @@ def make_tensor_divergence(bcs: Boundaries) -> OperatorType:
     assert isinstance(bcs.grid, CylindricalSymGrid)
     bcs.check_value_rank(1)
 
-    divergence_r = make_divergence(bcs.extract_component(0))
-    divergence_z = make_divergence(bcs.extract_component(1))
-    divergence_phi = make_divergence(bcs.extract_component(2))
+    # calculate preliminary quantities
+    dim_r, dim_z = bcs.grid.shape
+    rs = bcs.grid.axes_coords[0]
 
-    @jit_allocate_out(out_shape=(3,) + bcs.grid.shape)
+    # handle the boundary conditions
+    boundary_r, boundary_z = bcs
+    deriv_r_r = boundary_r.extract_component(0).make_derivative_evaluator()
+    deriv_r_z = boundary_r.extract_component(1).make_derivative_evaluator()
+    deriv_r_φ = boundary_r.extract_component(2).make_derivative_evaluator()
+    deriv_z_r = boundary_z.extract_component(0).make_derivative_evaluator()
+    deriv_z_z = boundary_z.extract_component(1).make_derivative_evaluator()
+    deriv_z_φ = boundary_z.extract_component(2).make_derivative_evaluator()
+
+    # use processing for large enough arrays
+    parallel = dim_r * dim_z >= config["numba.parallel_threshold"]
+
+    @jit_allocate_out(parallel=parallel, out_shape=(3,) + bcs.grid.shape)
     def tensor_divergence(arr, out=None):
-        """apply gradient operator to array `arr`"""
-        divergence_r(arr[0], out=out[0])
-        divergence_z(arr[1], out=out[1])
-        divergence_phi(arr[2], out=out[2])
+        """apply tensor divergence operator to array `arr`"""
+        # assign aliases
+        arr_rr, arr_rz, arr_rφ = arr[0, 0], arr[0, 1], arr[0, 2]
+        arr_zr, arr_zz, _ = arr[1, 0], arr[1, 1], arr[1, 2]
+        arr_φr, arr_φz, arr_φφ = arr[2, 0], arr[2, 1], arr[2, 2]
+        out_r, out_z, out_φ = out
+
+        for j in nb.prange(dim_z):  # iterate axial points
+            for i in range(dim_r):  # iterate radial points
+                derivs = deriv_z_r(arr_rz, (i, j)) + deriv_r_r(arr_rr, (i, j))
+                out_r[i, j] = derivs + (arr_rr[i, j] - arr_φφ[i, j]) / rs[i]
+
+                derivs = deriv_z_φ(arr_φz, (i, j)) + deriv_r_φ(arr_φr, (i, j))
+                out_φ[i, j] = derivs + (arr_rφ[i, j] + arr_φr[i, j]) / rs[i]
+
+                derivs = deriv_z_z(arr_zz, (i, j)) + deriv_r_z(arr_zr, (i, j))
+                out_z[i, j] = derivs + arr_zr[i, j] / rs[i]
+
         return out
 
     return tensor_divergence  # type: ignore
