@@ -122,7 +122,8 @@ def test_poisson_solver_polar():
 
         d = np.random.random(grid.shape)
         d -= ScalarField(grid, d).average  # balance the right hand side
-        np.testing.assert_allclose(laplace(poisson(d)), d, err_msg=f"bcs = {bc_val}")
+        res = laplace(poisson(d))
+        np.testing.assert_allclose(res, d, rtol=1e-6, err_msg=f"bcs = {bc_val}")
 
     grid = PolarSymGrid([2, 4], 8)
     for bc_val in ["natural", {"value": 1}]:
@@ -132,4 +133,60 @@ def test_poisson_solver_polar():
 
         d = np.random.random(grid.shape)
         d -= ScalarField(grid, d).average  # balance the right hand side
-        np.testing.assert_allclose(laplace(poisson(d)), d, err_msg=f"bcs = {bc_val}")
+        res = laplace(poisson(d))
+        np.testing.assert_allclose(res, d, rtol=1e-6, err_msg=f"bcs = {bc_val}")
+
+
+def test_examples_scalar_polar():
+    """compare derivatives of scalar fields for polar grids"""
+    grid = PolarSymGrid(1, 32)
+    sf = ScalarField.from_expression(grid, "r**3")
+
+    # gradient
+    res = sf.gradient([{"derivative": 0}, {"derivative": 3}])
+    expect = VectorField.from_expression(grid, ["3 * r**2", 0])
+    np.testing.assert_allclose(res.data, expect.data, rtol=0.1, atol=0.1)
+
+    # gradient squared
+    expect = ScalarField.from_expression(grid, "9 * r**4")
+    for c in [True, False]:
+        res = sf.gradient_squared([{"derivative": 0}, {"derivative": 3}], central=c)
+        np.testing.assert_allclose(res.data, expect.data, rtol=0.1, atol=0.1)
+
+    # laplace
+    res = sf.laplace([{"derivative": 0}, {"derivative": 3}])
+    expect = ScalarField.from_expression(grid, "9 * r")
+    np.testing.assert_allclose(res.data, expect.data, rtol=0.1, atol=0.1)
+
+
+def test_examples_vector_polar():
+    """compare derivatives of vector fields for polar grids"""
+    grid = PolarSymGrid(1, 32)
+    vf = VectorField.from_expression(grid, ["r**3", "r**2"])
+
+    # divergence
+    res = vf.divergence([{"derivative": 0}, {"value": 1}])
+    expect = ScalarField.from_expression(grid, "4 * r**2")
+    np.testing.assert_allclose(res.data, expect.data, rtol=0.1, atol=0.1)
+
+    # # vector Laplacian
+    # res = vf.laplace([{"derivative": 0}, {"value": 1}])
+    # expect = VectorField.from_expression(grid, ["8 * r", "3"])
+    # np.testing.assert_allclose(res.data, expect.data, rtol=0.1, atol=0.1)
+
+    # vector gradient
+    res = vf.gradient([{"derivative": 0}, {"value": [1, 1]}])
+    expr = [["3 * r**2", "-r"], ["2 * r", "r**2"]]
+    expect = Tensor2Field.from_expression(grid, expr)
+    np.testing.assert_allclose(res.data, expect.data, rtol=0.1, atol=0.1)
+
+
+def test_examples_tensor_polar():
+    """compare derivatives of tensorial fields for polar grids"""
+    grid = PolarSymGrid(1, 32)
+    tf = Tensor2Field.from_expression(grid, [["r**3"] * 2] * 2)
+
+    # tensor divergence
+    res = tf.divergence([{"derivative": 0}, {"value": [1, 1]}])
+    expect = VectorField.from_expression(grid, ["3 * r**2", "5 * r**2"])
+    np.testing.assert_allclose(res.data, expect.data, rtol=0.1, atol=0.1)
