@@ -51,7 +51,7 @@ class PDEBase(metaclass=ABCMeta):
     """ bool: Flag indicating whether the right hand side is a complex-valued PDE, which
     requires all involved variables to be of complex type """
 
-    def __init__(self, noise: ArrayLike = 0):
+    def __init__(self, noise: ArrayLike = 0, rng: np.random.Generator = None):
         """
         Args:
             noise (float or :class:`~numpy.ndarray`):
@@ -59,6 +59,10 @@ class PDEBase(metaclass=ABCMeta):
                 PDEs by default. If set to zero, a deterministic partial differential
                 equation will be solved. Different noise magnitudes can be supplied for
                 each field in coupled PDEs.
+            rng (:class:`~numpy.random.Generator`):
+                Random number generator (default: :func:`~numpy.random.default_rng()`).
+                Note that this random number generator is only used for numpy function,
+                while compiled numba code is unaffected.
 
         Note:
             If more complicated noise structures are required, the methods
@@ -69,6 +73,10 @@ class PDEBase(metaclass=ABCMeta):
         self._logger = logging.getLogger(self.__class__.__name__)
         self._cache: Dict[str, Any] = {}
         self.noise = noise
+        if rng is None:
+            self.rng = np.random.default_rng()
+        else:
+            self.rng = rng
 
     @property
     def is_sde(self) -> bool:
@@ -99,7 +107,7 @@ class PDEBase(metaclass=ABCMeta):
         """
 
         def modify_after_step(state_data: np.ndarray) -> float:
-            """ no-op function """
+            """no-op function"""
             return 0
 
         return modify_after_step
@@ -111,7 +119,7 @@ class PDEBase(metaclass=ABCMeta):
     def _make_pde_rhs_numba(
         self, state: FieldBase
     ) -> Callable[[np.ndarray, float], np.ndarray]:
-        """ create a compiled function for evaluating the right hand side """
+        """create a compiled function for evaluating the right hand side"""
         raise NotImplementedError
 
     def _make_pde_rhs_numba_cached(
@@ -201,7 +209,7 @@ class PDEBase(metaclass=ABCMeta):
             state = state.copy()
 
             def evolution_rate_numpy(state_data: np.ndarray, t: float) -> np.ndarray:
-                """ evaluate the rhs given only a state without the grid """
+                """evaluate the rhs given only a state without the grid"""
                 state.data = state_data
                 return self.evolution_rate(state, t).data
 
@@ -236,7 +244,7 @@ class PDEBase(metaclass=ABCMeta):
         if self.noise:
             if np.isscalar(self.noise):
                 # a single noise value is given for all fields
-                data = np.random.normal(scale=self.noise, size=state.data.shape)
+                data = self.rng.normal(scale=self.noise, size=state.data.shape)
                 return state.copy(data=data, label=label)
 
             elif isinstance(state, FieldCollection):
@@ -246,7 +254,7 @@ class PDEBase(metaclass=ABCMeta):
                     if n == 0:
                         data = 0
                     else:
-                        data = np.random.normal(scale=n, size=f.data.shape)
+                        data = self.rng.normal(scale=n, size=f.data.shape)
                     fields.append(f.copy(data=data))
                 return FieldCollection(fields, label=label)
 
@@ -281,7 +289,7 @@ class PDEBase(metaclass=ABCMeta):
 
                 @jit
                 def noise_realization(state_data: np.ndarray, t: float) -> np.ndarray:
-                    """ helper function returning a noise realization """
+                    """helper function returning a noise realization"""
                     return noise_strength * np.random.randn(*data_shape)  # type: ignore
 
             elif isinstance(state, FieldCollection):
@@ -293,7 +301,7 @@ class PDEBase(metaclass=ABCMeta):
 
                 @jit
                 def noise_realization(state_data: np.ndarray, t: float) -> np.ndarray:
-                    """ helper function returning a noise realization """
+                    """helper function returning a noise realization"""
                     out = np.random.randn(*data_shape)
                     for i in range(data_shape[0]):
                         # TODO: Avoid creating random numbers when noise_strengths == 0
@@ -310,7 +318,7 @@ class PDEBase(metaclass=ABCMeta):
 
             @jit
             def noise_realization(state_data: np.ndarray, t: float) -> None:
-                """ helper function returning a noise realization """
+                """helper function returning a noise realization"""
                 return None
 
         return noise_realization  # type: ignore
@@ -333,7 +341,7 @@ class PDEBase(metaclass=ABCMeta):
 
         @jit
         def sde_rhs(state_data: np.ndarray, t: float) -> Tuple[np.ndarray, np.ndarray]:
-            """ compiled helper function returning a noise realization """
+            """compiled helper function returning a noise realization"""
             return (evolution_rate(state_data, t), noise_realization(state_data, t))
 
         return sde_rhs  # type: ignore
@@ -401,7 +409,7 @@ class PDEBase(metaclass=ABCMeta):
             def sde_rhs(
                 state_data: np.ndarray, t: float
             ) -> Tuple[np.ndarray, np.ndarray]:
-                """ evaluate the rhs given only a state without the grid """
+                """evaluate the rhs given only a state without the grid"""
                 state.data = state_data
                 return (
                     self.evolution_rate(state, t).data,
