@@ -111,7 +111,16 @@ class PDE(PDEBase):
         if not isinstance(rhs, OrderedDict):
             rhs = OrderedDict(rhs)
         if "t" in rhs:
-            raise RuntimeError("`t` is not allowed as a variable since it denotes time")
+            raise ValueError("Cannot name field `t` since it denotes time")
+        reserved_symbols = set(rhs) & ScalarExpression._reserved_symbols
+        if any(reserved_symbols):
+            if len(reserved_symbols) == 1:
+                name = reserved_symbols.pop()
+                raise ValueError(f"Cannot use reserved symbol `{name}` as field name")
+            else:
+                raise ValueError(
+                    f"Cannot use reserved symbols {reserved_symbols} as field names"
+                )
         if consts is None:
             consts = {}
 
